@@ -5,8 +5,9 @@ Verilog Codegen
 import itertools
 import logging
 import warnings
-from typing import Generator, cast
+from typing import Generator, Iterator, cast
 
+from python2verilog.backend.verilog.config import TestbenchConfig
 from python2verilog.optimizer.optimizer import backwards_replace
 from python2verilog.utils.lines import Lines
 
@@ -83,7 +84,7 @@ class CodeGen:
             str_ += ");"
             return str_
 
-        def create_instance_zeroed_signals() -> Generator[ver.Statement, None, None]:
+        def create_instance_zeroed_signals() -> Iterator[ver.Statement]:
             """
             Instance signals that should always be set to zero be default
             """
@@ -202,19 +203,12 @@ class CodeGen:
             key: ir.UInt(index) for index, key in enumerate(sorted(context.states))
         }
 
-        python_test_code = Lines()
-        for case in context.test_cases:
-            python_test_code += f"print(list({context.name}(*{case})))"
         return ver.Module(
             name=context.name,
             inputs=inputs,
             outputs=outputs,
             body=body,
             localparams=state_vars,
-            header=Lines(
-                f"/*\n\n# Python Function\n{context.py_string}\n\n"
-                f"# Test Cases\n{python_test_code}\n*/\n\n"
-            ),
         )
 
     @staticmethod
@@ -233,14 +227,17 @@ class CodeGen:
             # The first case can be included here
             # Known as Quick Start
             mapping = {
-                ir.Expression(var.ver_name): ir.Expression(var.py_name)
+                ir.Var(py_name=var.ver_name, ver_name=var.ver_name): ir.Expression(
+                    var.py_name
+                )
                 for var in context.input_vars
             }
 
             for var in context.input_vars:
                 then_body.append(
                     ver.NonBlockingSubsitution(
-                        ir.Expression(var.ver_name), ir.Expression(var.py_name)
+                        ir.Var(py_name=var.ver_name, ver_name=var.ver_name),
+                        ir.Expression(var.py_name),
                     )
                 )
 
@@ -266,7 +263,8 @@ class CodeGen:
             for var in context.input_vars:
                 then_body.append(
                     ver.NonBlockingSubsitution(
-                        ir.Expression(var.ver_name), ir.Expression(var.py_name)
+                        ir.Var(py_name=var.ver_name, ver_name=var.ver_name),
+                        ir.Expression(var.py_name),
                     )
                 )
             then_body.append(
@@ -314,14 +312,15 @@ class CodeGen:
         """
         return self.get_module_lines().to_string()
 
-    def get_testbench(self, random_wait: bool = False):
+    def get_testbench(self, config: TestbenchConfig):
         """
         Creates testbench with multiple test cases
 
         Each element of self.context.test_cases represents a single test case
 
-        :param random_wait: whether or not to have random _wait signal in the while loop
+        :param random_ready: whether or not to have random ready signal in the while loop
         """
+        logging.debug(f"{config}")
         if len(self.context.input_vars) == 0:
             raise RuntimeError(
                 f"Input var names not deduced for {self.context.name}, "
@@ -335,13 +334,13 @@ class CodeGen:
 
         def make_display_stmt():
             """
-            Creates a display statement for valid + outputs
+            Creates a display statement for protocol signals and outputs
 
-            $display("%0d, ...", _valid, ...);
+            $display("%0d, ...", ...);
             """
             string = '$display("%0d, %0d, '
             string += "%0d, " * (len(self.context.output_vars) - 1)
-            string += '%0d", _valid, _ready'
+            string += '%0d", _ready, _valid'
             for var in self.context.output_vars:
                 string += f", {var.ver_name}"
             string += ");"
@@ -392,7 +391,8 @@ class CodeGen:
             for i, var in enumerate(self.context.input_vars):
                 initial_body.append(
                     ver.BlockingSub(
-                        ir.Expression(var.py_name), ir.Int(int(test_case[i]))
+                        ir.Var(py_name=var.py_name, ver_name=var.py_name),
+                        ir.Int(int(test_case[i])),
                     )
                 )
             initial_body.append(ver.BlockingSub(self.context.signals.start, ir.UInt(1)))
@@ -403,7 +403,7 @@ class CodeGen:
             for i, var in enumerate(self.context.input_vars):
                 initial_body.append(
                     ver.BlockingSub(
-                        ir.Expression(var.py_name),
+                        ir.Var(py_name=var.py_name, ver_name=var.py_name),
                         ir.Unknown(),
                         comment="only need inputs when start is set",
                     )
@@ -414,7 +414,7 @@ class CodeGen:
             # While loop waitng for ready signal
             while_body: list[ver.Statement] = []
             # while_body.append(make_display_stmt())
-            if random_wait:
+            if config.random_ready:
                 while_body.append(ver.Statement("_ready = $urandom_range(0, 4) === 0;"))
             while_body.append(
                 ver.Statement(
@@ -453,6 +453,9 @@ class CodeGen:
 
         initial_loop = ver.Initial(body=initial_body)
 
+        python_test_code = Lines()
+        for case in self.context.test_cases:
+            python_test_code += f"print(list({self.context.name}(*{case})))"
         if self.context:
             module = ver.Module(
                 self.context.testbench_name,
@@ -460,21 +463,25 @@ class CodeGen:
                 [],
                 body=setups + [initial_loop],
                 is_not_testbench=False,
+                header=Lines(
+                    f"/*\n\n# Python Function\n{self.context.py_string}\n\n"
+                    f"# Test Cases\n{python_test_code}\n*/\n\n"
+                ),
             )
             return module
         raise RuntimeError("Needs the context")
 
-    def get_testbench_lines(self, random_wait: bool = False):
+    def get_testbench_lines(self, config: TestbenchConfig):
         """
         New Testbench as lines
         """
-        return self.get_testbench(random_wait=random_wait).to_lines()
+        return self.get_testbench(config=config).to_lines()
 
-    def get_testbench_str(self, random_wait: bool = False):
+    def get_testbench_str(self, config: TestbenchConfig):
         """
         New testbench as str
         """
-        return self.get_testbench_lines(random_wait=random_wait).to_string()
+        return self.get_testbench_lines(config=config).to_string()
 
 
 class CaseBuilder:
@@ -545,6 +552,7 @@ class CaseBuilder:
         """
         Processes a node
         """
+        logging.debug(f"{self.do_vertex.__name__} {vertex} {len(self.visited)}")
 
         assert isinstance(vertex, ir.Node), str(vertex)
         self.visited.add(vertex.unique_id)
