@@ -10,7 +10,9 @@ from .expressions import (
     Add,
     BinOp,
     Div,
+    ExclusiveVar,
     Expression,
+    FloorDiv,
     Int,
     LessThan,
     Mod,
@@ -33,8 +35,8 @@ from .graph import (
     Edge,
     Element,
     IfElseNode,
+    Node,
     NonClockedEdge,
-    Vertex,
     YieldNode,
     create_cytoscape_elements,
     create_networkx_adjacency_list,
